from textwrap import dedent

from mtmai.agents.crawai.types import Chapter
from mtmai.crewai import Agent, Crew, Process, Task
from mtmai.crewai.llm import LLM


class WriteBookChapterCrew:
    """Write Book Chapter Crew"""

    def __init__(self, llm: LLM):
        self.llm = llm

    def crew(self) -> Crew:
        """Creates the Write Book Chapter Crew"""

        researcher_agent = Agent(
            role="Research Agent",
            goal=dedent("""Gather comprehensive information about {topic} and {chapter_title} that will be used to enhance the content of the chapter.
    Here is some additional information about the author's desired goal for the book and the chapter:\n\n {goal}
    Here is the outline description for the chapter:\n\n {chapter_description}"""),
            backstory=dedent("""You are an experienced researcher skilled in finding the most relevant and up-to-date information on any given topic.
    Your job is to provide insightful data that supports and enriches the writing process for the chapter."""),
            tools=[],
            llm=self.llm,
        )

        writer_agent = Agent(
            role="Chapter Writer",
            goal=dedent("""Write a well-structured chapter for the book based on the provided chapter title, goal, and outline.
    The chapter should be written in markdown format and contain around 3,000 words."""),
            backstory=dedent("""You are an exceptional writer, known for producing engaging, well-researched, and informative content.
    You excel at transforming complex ideas into readable and well-organized chapters."""),
            llm=self.llm,
        )

        research_chapter_task = Task(
            description=dedent("""Research the provided chapter topic, title, and outline to gather additional content that will be helpful in writing the chapter.
    Ensure you focus on reliable, high-quality sources of information.

    Here is some additional information about the author's desired goal for the book and the chapter:\n\n {goal}
    Here is the outline description for the chapter:\n\n {chapter_description}

    When researching, consider the following key points:
    - you need to gather enough information to write a 3,000-word chapter
    - The chapter you are researching needs to fit in well with the rest of the chapters in the book.

    Here is the outline of the entire book:\n\n
    {book_outline}"""),
            expected_output="A set of additional insights and information that can be used in writing the chapter.",
            agent=researcher_agent,
        )

        write_chapter_task = Task(
            description=dedent("""Write a well-structured chapter based on the chapter title, goal, and outline description.
    Each chapter should be written in markdown and should contain around 3,000 words.

    Here is the topic for the book: {topic}
    Here is the title of the chapter: {chapter_title}
    Here is the outline description for the chapter:\n\n {chapter_description}

    Important notes:
    - The chapter you are writing needs to fit in well with the rest of the chapters in the book.

    Here is the outline of the entire book:\n\n
    {book_outline}"""),
            agent=writer_agent,
            expected_output="A markdown-formatted chapter of around 3,000 words that covers the provided chapter title and outline description.",
            output_pydantic=Chapter,
        )
        return Crew(
            agents=[researcher_agent, writer_agent],
            tasks=[research_chapter_task, write_chapter_task],
            process=Process.sequential,
            verbose=True,
        )
